@@ -1,4 +1,5 @@
 import logging
+from pathlib import Path
 
 from esi.clients import EsiClientProvider
 
@@ -7,4 +8,7 @@ from app_utils.logging import LoggerAddTag
 from . import __title__, __version__
 
 logger = LoggerAddTag(logging.getLogger(__name__), __title__)
-esi = EsiClientProvider(app_info_text=f"aa-structures v{__version__}")
+swagger_path = Path(__file__).parent / "swagger.json"
+esi = EsiClientProvider(
+    app_info_text=f"aa-structures v{__version__}", spec_file=swagger_path
+)
